% Copyright (c) 2011, Tom Van Cutsem, Vrije Universiteit Brussel
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in the
%      documentation and/or other materials provided with the distribution.
%    * Neither the name of the Vrije Universiteit Brussel nor the
%      names of its contributors may be used to endorse or promote products
%      derived from this software without specific prior written permission.
%
%THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
%ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
%WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
%DISCLAIMED. IN NO EVENT SHALL VRIJE UNIVERSITEIT BRUSSEL BE LIABLE FOR ANY
%DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
%(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES%
%LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
%ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
%(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
%SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% pmap implementation based on Joe Armstrong's "Programming Erlang" book

-module(pmap).
-export([pmap/2, tests/1, fib/1]).

pmap(F, L) ->
S = self(),
	Ref = erlang:make_ref(), % creates a unique reference
	Pids = lists:map(fun(I) -> spawn_link(fun() -> do_f(S, Ref, F, I) end) end, L),
	gather(Pids, Ref).

do_f(Parent, Ref, F, I) ->
	Parent ! { self(), Ref, F(I) }.

gather([Pid|T], Ref) ->
	receive
    {Pid, Ref, Val} -> [Val | gather(T, Ref)]
  end;
gather([], _) -> [].

tests([N]) -> % N = command line argument
  Nsched = list_to_integer(atom_to_list(N)),
	run_tests(1, Nsched).

run_tests(T, Nsched) ->
  case test(T) of
	    stop -> init:stop();
	    {Benchmark, TSeq, TPar} ->
	      io:format("~p,~p,~p~n", [Nsched,Benchmark,TSeq / TPar]),
	      run_tests(T+1, Nsched)
  end.

test(1) ->
 %% Make 100 lists
 %%	Each list contains 1000 random integers
 seed(),
 S = lists:seq(1,100),
 L = lists:map(fun(_) -> mkList(1000) end, S),
 {Time1, Sorted} = timer:tc(lists,	map, [fun lists:sort/1, L]),
 {Time2, Sorted} = timer:tc(pmap, pmap, [fun lists:sort/1, L]),
 {sort, Time1, Time2};
test(2) ->
 %% L = [27,27,27,..] 100 times
 L = lists:duplicate(100, 27),
 {Time1, Res} = timer:tc(lists,	map, [fun fib/1, L]),
 {Time2, Res} = timer:tc(pmap, pmap, [fun fib/1, L]),
 {fib, Time1, Time2};
test(3) -> stop.

%% inefficient fibonacci
fib(0) -> 1;
fib(1) -> 1;
fib(N) -> fib(N-1) + fib(N-2).

%% Reset the random number generator. This is so we
%% get the same sequence of random numbers each time we run
%% the program
seed() -> random:seed(44,55,66).

%% Make a list of K random numbers
%%	Each random number in the range 1..1000000
mkList(K) -> mkList(K, []).
mkList(0, L) -> L; mkList(N, L) -> mkList(N-1, [random:uniform(1000000)|L]).
