% Copyright (c) 2011-2012, Tom Van Cutsem, Vrije Universiteit Brussel
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in the
%      documentation and/or other materials provided with the distribution.
%    * Neither the name of the Vrije Universiteit Brussel nor the
%      names of its contributors may be used to endorse or promote products
%      derived from this software without specific prior written permission.
%
%THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
%ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
%WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
%DISCLAIMED. IN NO EVENT SHALL VRIJE UNIVERSITEIT BRUSSEL BE LIABLE FOR ANY
%DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
%(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES
%LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
%ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
%(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
%SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% Process ring Benchmark:
% from the book "Erlang Programming" by Francesco Cesarini

% spawn N processes that each send a message to their successor
% can benchmark using timer:tc
% on my macbook pro machine: spawn approx. 1 million processes / sec

-module(ring_benchmark).
-export([construct_ring/1, ring_process/2]).

construct_ring(N) ->
  ring_process(N, self()).

ring_process(0, First) ->
  First ! ok;
ring_process(N, First) ->
  Succ = spawn(?MODULE, ring_process, [N-1, First]),
	Succ ! ok,
	receive
	  ok -> ok
	end.

% timer:tc(ring_benchmark, construct_ring, [10]).
%{239,ok}
% timer:tc(ring_benchmark, construct_ring, [100]).
%{281,ok}
% timer:tc(ring_benchmark, construct_ring, [1000]).
%{2473,ok}
% timer:tc(ring_benchmark, construct_ring, [10000]).
%{16316,ok}
% timer:tc(ring_benchmark, construct_ring, [100000]).
%{110250,ok}
% timer:tc(ring_benchmark, construct_ring, [1000000]).
%{1026293,ok}
% timer:tc(ring_benchmark, construct_ring, [10000000]).
%{10169272,ok}