% Copyright (c) 2011, Tom Van Cutsem, Vrije Universiteit Brussel
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in the
%      documentation and/or other materials provided with the distribution.
%    * Neither the name of the Vrije Universiteit Brussel nor the
%      names of its contributors may be used to endorse or promote products
%      derived from this software without specific prior written permission.
%
%THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
%ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
%WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
%DISCLAIMED. IN NO EVENT SHALL VRIJE UNIVERSITEIT BRUSSEL BE LIABLE FOR ANY
%DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
%(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES%
%LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
%ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
%(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
%SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

-module(stopwatch).
-export([new/0, start/1, stop/1, pause/1, idle/0]).

% A simple stopwatch process that can be started, stopped and paused.
% Illustrates how processes in Erlang can be used to encode finite
% state machines.

% now(), see http://www.erlang.org/doc/man/erlang.html#now-0
% timer module, see http://www.erlang.org/doc/man/timer.html

% interface functions
new() ->
  spawn(stopwatch, idle, []).

start(Watch) ->
  Watch ! start.
stop(Watch) ->
  Watch ! {self(), stop},
  receive
    {Watch, Elapsed} -> Elapsed
  after timer:seconds(2) ->
    unresponsive
  end.
pause(Watch) ->
  Watch ! pause.

% implementation
idle() ->
  receive
    start ->
      running(now(), 0);
    Any ->
      io:format("invalid message when idle: ~w~n", [Any]),
      idle()
  end.

running(Start, Elapsed) ->
  receive
    {From, stop} ->
      From ! { self(), timer:now_diff(now(), Start) + Elapsed },
      idle();
    pause ->
      paused(timer:now_diff(now(), Start) + Elapsed);
    Any ->
      io:format("invalid message when running: ~w~n", [Any]),
      running(Start, Elapsed)
  end.
  
paused(Elapsed) ->
  receive
    start ->      
      running(now(), Elapsed);      
    Any ->
      io:format("invalid message when paused: ~w~n", [Any]),
      paused(Elapsed)
  end.